<?php

/*
 * This is the kirbyMVC core
 * The core provides all the magic behind kirbyMVC
 */

//Include, most important, kirby!
require_once('kirby.php');

//Include globals and the configuration file!
require_once('globals.php');
require_once('config.php');

//Include all available plugins!
$plugins = dir::read('includes/plugins');
foreach ($plugins as $plugin) {
	require_once('includes/plugins/' . $plugin);
}

//Custom extract function
function _extract($array) {
	foreach ($array as $key => $value) {
		tpl::set($key, $value);
	}
}

//Include the main model and controller
require_once('includes/model.php');
require_once('includes/controller.php');

//All the core is going to do is to load the model and controller according to the URL
$controller = null;
$action = null;
$params = array();
$url = r::get('url');

//Note: Rewriting is very important!
if (r::get('controller') || r::get('action') || r::get('params')) {
	//mod_rewrite ist not enabled
	$controller = r::get('controller');
	$action = r::get('action');
} else {
	if (!empty($url)) {
		$url = explode('/', $url);
		if (isset($url[0])) { $controller = $url[0]; }
		if (isset($url[1])) { $action = $url[1]; }
		for ($i = 2; $i < count($url); $i++) {
			$kv_pair = explode(':', $url[$i]);
			if (count($kv_pair) == 2) {
				$params[$kv_pair[0]] = $kv_pair[1];
			} else {
				array_push($params, $kv_pair);
			}
		}
	}
}

//Setup some template variables that have to be permanently available
tpl::set('path', c::get('path'));

if ( $controller ) {
	//If the controller get value has been provided
	
	if ( 	file_exists( 'data/models/' . $controller . '_model.php' ) &&
			file_exists( 'data/controllers/' . $controller . '_controller.php' ) &&
			file_exists( 'data/views/' . $controller )
	) {
		//If the provided controller, model and view exists

		require_once('data/models/' . $controller . '_model.php');
		require_once('data/controllers/' . $controller . '_controller.php');
		
		$model_instance = $controller . 'Model';
		$model_instance = new $model_instance();
		$controller_instance = $controller . 'Controller';
		$controller_instance = new $controller_instance( $model_instance );
		
		$controller_instance->params = $params;
		
		if ( $action ) {
			//If the action get value has been provided
			
			if ( file_exists( 'data/views/' . $controller . '/' . $action . '.php' ) ) {
				//If the view for the provided action exists
				
				tpl::set('content', $controller . '/' . $action);
				$controller_instance->$action();
				_extract( $controller_instance->sets );
				
			} else {
				//Otherwise the desired view could not be found
				tpl::set('content', '404');
				
			}
			
		} else {
			//Otherwise execute the index method
			
			tpl::set('content', $controller . '/index');
			$controller_instance->index();
			_extract( $controller_instance->sets );
			
		}
		
	} else {
		//Otherwise the desired content could not be found
		
		tpl::set('content', '404');
		
	}

	
} else {
	//Otherwise the index view has to be displayed
	
	require_once('data/models/default_model.php');
	require_once('data/controllers/default_controller.php');
	
	$model_instance = new defaultModel();
	$controller_instance = new defaultController( $model_instance );
	
	$controller_instance->params = $params;

	tpl::set('content', 'default/index');
	$controller_instance->index();
	_extract( $controller_instance->sets );

}

?>